/***************************************************************************/
/*                                                                         */
/* File: collision.h                                                       */
/*                                                                         */
/* Author: Ben Kenwright                                                   */
/*                                                                         */
/* Email: bkenwright@xbdev.net                                             */
/*                                                                         */
/* URL: www.xbdev.net                                                      */
/*                                                                         */
/* Date: 26-08-05                                                          */
/*                                                                         */
/***************************************************************************/
/*
   Various simple collision detection algoriths
*/
/***************************************************************************/

#ifndef COLLISION_H
#define COLLISION_H


#include <d3dx9.h>					// DirectX Header File

#pragma comment(lib, "d3d9.lib")	// DirectX Library Files
#pragma comment(lib, "d3dx9.lib")


/***************************************************************************/
/*                                                                         */
/* class CTri                                                              */
/*                                                                         */
/***************************************************************************/

class CTri
{
	public:
	// 3 Points which make up the corners of a triangle
	D3DXVECTOR3 p0,p1,p2;

	CTri(D3DXVECTOR3& a0, D3DXVECTOR3& a1, D3DXVECTOR3& a2)
	{
		p0=a0;
		p1=a1;
		p2=a2;
	}

	CTri(){};
};

/***************************************************************************/
/*                                                                         */
/* class CLine                                                             */
/*                                                                         */
/***************************************************************************/

class CLine
{
	public:
	// 2 Points which make up each end of the line
	D3DXVECTOR3 p0;
	D3DXVECTOR3 p1;

	CLine(D3DXVECTOR3& a0, D3DXVECTOR3& a1)
	{
		p0=a0;
		p1=a1;
	}

	CLine(){}
};


/***************************************************************************/
/*                                                                         */
/*  Collision / Intersection Functions                                     */
/*                                                                         */
/***************************************************************************/

// Probably the fastest method and uses barycentric coordinates, and is the
// algorithm put forward by Moller and Trumbore
bool TriangleLineCollision(D3DXVECTOR3 p0,	 // Start of our line
						   D3DXVECTOR3 p1,	 // End point of our line
						   D3DXVECTOR3 pA,	 // |
						   D3DXVECTOR3 pB,   //  > 3 Points for our triangle
						   D3DXVECTOR3 pC,   // |
						   D3DXVECTOR3* hit);// Hit point on triangle



// Alternative version, but uses a different method thats a little slower
// but easier to understand algorithm.
bool TriangleLineCollision2(D3DXVECTOR3 p, D3DXVECTOR3 q,
						    D3DXVECTOR3 a, D3DXVECTOR3 b, D3DXVECTOR3 c);


inline bool PointInTriangle(D3DXVECTOR3& pntA,
			   				D3DXVECTOR3& a, D3DXVECTOR3& b, D3DXVECTOR3& c);

inline float DistRayPlane(D3DXVECTOR3& rayOrigin,
                          D3DXVECTOR3& rayVector,
                          D3DXVECTOR3& planeNormal,
                          float planeD);

#endif // COLLISION_H

