/***************************************************************************/
/*                                                                         */
/*  File: game.cpp                                                         */
/*  Date: 26-08-06                                                         */
/*                                                                         */
/***************************************************************************/
/*
    Updated: xx-xx-xx
*/
/***************************************************************************/


#include <math.h>					// cosf(..) sinf(..), sqrt(..)
#include <stdio.h>					// sprintf(..)

#include <windows.h>				// GetKeyState(..)

#include <d3dx9.h>					// DirectX Header File

#pragma comment(lib, "d3d9.lib")	// DirectX Library Files
#pragma comment(lib, "d3dx9.lib")


#include "common.h"					// SafeRelease(..)

#include "collision.h"				// Collision Algorithms / Container Classes

#include "dxdbg.h"					// Various DirectX Debug

									// DirectX Device/Render Handles
extern LPDIRECT3D9			g_pD3D;			// main.cpp
extern LPDIRECT3DDEVICE9	g_pD3DDevice;	// main.cpp

extern bool					g_LMousePress;	// main.cpp
extern int					g_mousePosX;	// main.cpp
extern int					g_mousePosY;	// main.cpp

/***************************************************************************/
/*                                                                         */
/*  Globals                                                                */
/*                                                                         */
/***************************************************************************/

static bool			g_bWire			= false;  // Wire frame rendering on/off

static CTri			g_tri			= CTri( D3DXVECTOR3(1,-1,-1),
											D3DXVECTOR3(0, 1, 0),
											D3DXVECTOR3(-1,-1, 1));

static CLine		g_line			= CLine( D3DXVECTOR3(0.5f,0,0),
											 D3DXVECTOR3(1.5f,0,0));

static bool			g_collision		= false;  // Simple variable to check if
											  // have a collision

											  // Record the hit pos if any
static D3DXVECTOR3	g_hitPos		= D3DXVECTOR3(0,0,0);

/***************************************************************************/
/*                                                                         */
/*  Name: KeyBoard()                                                       */
/*  Date: 03-12-05                                                         */
/*                                                                         */
/*  Used to accept keyboard input and alter some global variables - used   */
/*  for simple analysis of our code - turning wire frame rendering on/off  */
/*  adjusting variables which are used in the generation of our shape etc. */
/*                                                                         */
/***************************************************************************/
void KeyBoard()
{
	// Controlling the rendering
	if( GetKeyState('W')		& 0x80 ) g_bWire=!g_bWire;


	const float speed = 0.05f;
	D3DXVECTOR3 move = D3DXVECTOR3(0,0,0);
	
	if ( GetKeyState(VK_UP)		& 0x80 ) move.y += speed;
	if ( GetKeyState(VK_DOWN) 	& 0x80 ) move.y -= speed;

	if ( GetKeyState(VK_RIGHT) 	& 0x80 ) move.x += speed;
	if ( GetKeyState(VK_LEFT) 	& 0x80 ) move.x -= speed;

	if ( GetKeyState(0xBC) 		& 0x80 ) move.z += 0.5f*speed; // Key '>'
	if ( GetKeyState(0xBE) 		& 0x80 ) move.z -= 0.5f*speed; // Key '<'

	g_line.p0 += move;
	g_line.p1 += move;

	// Little bit of code so I could find out the > and < keys
	/*
	int ii = 0; 
	for (int ii=0; ii<500; ii++) 
	{
		if ( GetKeyState(ii) & 0x80 )
		{
			dprintf("key: 0x%X\n", ii);
		}
	}
	*/

}//End KeyBoard()


/***************************************************************************/
/*                                                                         */
/*  Name: SetCamera(..)                                                    */
/*                                                                         */
/***************************************************************************/

void SetCamera()
{
	D3DXVECTOR3 vCamPosition;
	static D3DXVECTOR3 vCamRot = D3DXVECTOR3(0,0,0);
	static D3DXVECTOR3 vCamRotOld = vCamRot;
	static D3DXVECTOR3 vCamDir = D3DXVECTOR3(0,4,-2);

	D3DXMATRIX mr;
	D3DXMatrixRotationYawPitchRoll(&mr, vCamRot.y, vCamRot.x, vCamRot.z);
	D3DXVec3TransformCoord(&vCamPosition, &vCamDir, &mr);

	if (g_LMousePress)
	{
		vCamRot.y = vCamRotOld.y + 0.01f*g_mousePosX;
		vCamRot.x = vCamRotOld.x + 0.01f*g_mousePosY;
	}
	else
	{
		vCamRotOld = vCamRot;
	}


    // [1] D3DTS_VIEW
    D3DXMATRIX v;
    g_pD3DDevice->SetTransform(	D3DTS_VIEW, 
								D3DXMatrixLookAtLH(&v, 
								&vCamPosition, 
                                &D3DXVECTOR3(0,0,0), 
                                &D3DXVECTOR3(0,1,0)));  
    // [2] D3DTS_PROJECTION
    D3DXMATRIX p;
    g_pD3DDevice->SetTransform( D3DTS_PROJECTION, 
								D3DXMatrixPerspectiveFovLH( &p,  
								D3DX_PI/4,  
								1.0f,
                                1.0f,500.0f));
}//End SetCamera(..)



/***************************************************************************/
/*                                                                         */
/*  Name: SetMaterial(..)                                                  */
/*                                                                         */
/***************************************************************************/

void SetMaterial()
{
      D3DMATERIAL9 matMaterial;
      D3DCOLORVALUE rgbaDiffuse  = {1.0, 1.0, 1.0, 0.0};  //RGBA
      D3DCOLORVALUE rgbaAmbient  = {1.0, 1.0, 1.0, 0.0};
      D3DCOLORVALUE rgbaSpecular = {0.0, 0.0, 0.0, 0.0};
      // emissive is for a glow effect
      D3DCOLORVALUE rgbaEmissive = {0.0, 0.0, 0.0, 0.0};

      matMaterial.Diffuse  = rgbaDiffuse;
      matMaterial.Ambient  = rgbaAmbient;
      matMaterial.Specular = rgbaSpecular;
      matMaterial.Emissive = rgbaEmissive;
      matMaterial.Power    = 100.0f;

      g_pD3DDevice->SetMaterial(&matMaterial);
}//End SetMaterial(..)



/***************************************************************************/
/*                                                                         */
/*  Name: SetLight(..)                                                     */
/*                                                                         */
/***************************************************************************/

void SetLight()
{
	//Set a light up?
    D3DLIGHT9 d3dLight;

    //Initialize the light structure.
    ZeroMemory(&d3dLight, sizeof(D3DLIGHT9));

    //Set up a white point light at (0, 0, -10).
    d3dLight.Type			= D3DLIGHT_POINT;

    // Actual colour of the light. (e.g. white).
    d3dLight.Diffuse.r		= 1.0f;
    d3dLight.Diffuse.g		= 1.0f;
    d3dLight.Diffuse.b		= 1.0f;

    // Colour when no light hits the object.
    d3dLight.Ambient.r		= 0.2f;
    d3dLight.Ambient.g		= 0.2f;
    d3dLight.Ambient.b		= 0.2f;

    // Shiny effect.
    d3dLight.Specular.r		= 0.1f;
    d3dLight.Specular.g     = 0.1f;
    d3dLight.Specular.b     = 0.1f;

    d3dLight.Position.x     = 0.0f;
    d3dLight.Position.y     = 0.0f;
    d3dLight.Position.z     = -30.0f;

    d3dLight.Attenuation0	= 1.0f; 
    d3dLight.Attenuation1	= 0.0f; 
    d3dLight.Attenuation2	= 0.0f; 
    d3dLight.Range			= 100.0f;

    g_pD3DDevice->SetLight(0, &d3dLight);
    g_pD3DDevice->LightEnable(0,FALSE);

    //Turn on lighting
    g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pD3DDevice->SetRenderState(D3DRS_AMBIENT, 
								 D3DCOLOR_XRGB(20,20,20));
}//End SetLight(..)



/***************************************************************************/
/*                                                                         */
/*  Name: DrawTriangle(..)                                                 */
/*                                                                         */
/***************************************************************************/
void DrawTriangle(D3DXVECTOR3& p0, D3DXVECTOR3& p1, D3DXVECTOR3& p2)
{
	struct stV
	{
		D3DXVECTOR3 p; //Position
		D3DXVECTOR3 n; //Normal
		DWORD       c; //Colour
		enum{ FVF_V = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE };
	};

	//g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	//g_pD3DDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
	//g_pD3DDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);

	stV verts[3];
	verts[0].p = p0;
	verts[1].p = p1;
	verts[2].p = p2;

	verts[0].c = 0xff0000ff;
	verts[1].c = 0xff00ff00;
	verts[2].c = 0xffff0000;

	D3DXVECTOR3 v0 = p1 - p0;
	D3DXVECTOR3 v1 = p2 - p0;
	D3DXVECTOR3 norm;
	D3DXVec3Cross(&norm, &v0, &v1);
	D3DXVec3Normalize(&norm, &norm);
	verts[0].n = verts[1].n = verts[2].n = -norm;

	const static int numTriangles = 1;

	g_pD3DDevice->SetTexture( 0, NULL);
	g_pD3DDevice->SetIndices(NULL);


	#if(DIRECT3D_VERSION >= 0x0900)
		g_pD3DDevice->SetFVF( stV::FVF_V);
	#else
		g_pD3DDevice->SetVertexShader( stV::FVF_V );
	#endif // DIRECT3D_VERSION

	//Render our data to the directx device
	g_pD3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLELIST, 
								   numTriangles, 
								   verts, 
								   sizeof( stV ) );

}// End DrawTriangle(..)


/***************************************************************************/
/*                                                                         */
/*  Name: DrawLine3D(..)                                                   */
/*                                                                         */
/***************************************************************************/

void DrawLine3D(D3DXVECTOR3& p0, D3DXVECTOR3& p1)
{
	struct stV
	{
		D3DXVECTOR3 p; //Position
		DWORD       c; //Colour
		enum{ FVF_V = D3DFVF_XYZ  | D3DFVF_DIFFUSE };
	};

	//g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	//g_pD3DDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
	//g_pD3DDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);

	stV verts[3];
	verts[0].p = p0;
	verts[1].p = p1;

	verts[0].c = 0xffffffff;
	verts[1].c = 0xffffffff;

	const static int numLines = 1;

	g_pD3DDevice->SetTexture( 0, NULL);

	#if(DIRECT3D_VERSION >= 0x0900)
		g_pD3DDevice->SetFVF( stV::FVF_V);
	#else
		g_pD3DDevice->SetVertexShader( stV::FVF_V );
	#endif // DIRECT3D_VERSION

		
	//Render our data to the directx device
	g_pD3DDevice->DrawPrimitiveUP( D3DPT_LINELIST, 
								   numLines, 
								   verts, 
								   sizeof( stV ) );

}// End DrawLine3D(..)


/*
void DrawLine2DThick(D3DXVECTOR2& p0, D3DXVECTOR2& p1)
{
	D3DXVECTOR2 vLine[2];
	ID3DXLine* pLine;

	pLine->SetWidth( 3 );
	pLine->SetAntialias( false );
	pLine->SetGLLines( true );

	vLine[0] = p0;
	vLine[1] = p1;

	pLine->Begin( );
	pLine->Draw(	vLine,			// Line Points
					2,				// Num Verts
					D3DCOLOR_RGBA(  // Col - 
						0xff,		//     - Red
						0xff,		//	   - Green
						0,			//     - Blue
						0xff ) );	//     - Alpha

	pLine->End( );
}
*/

/***************************************************************************/
/*                                                                         */
/*  Name: DrawShapes(..)                                                   */
/*                                                                         */
/***************************************************************************/
void DrawShape()
{
	DWORD val;
	g_pD3DDevice->GetRenderState(D3DRS_CULLMODE, &val);
	if (val != D3DCULL_NONE)
		g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	DrawTriangle(g_tri.p0, g_tri.p1, g_tri.p2);

	DrawLine3D(g_line.p0, g_line.p1);
}//End DrawShape(..)


/***************************************************************************/
/*                                                                         */
/* GameLoop()                                                              */
/* Our main render loop, which gets called over and over agian to do our   */
/* drawing...clears the screen, draws the data, then presents it..         */
/*                                                                         */
/***************************************************************************/
void GameLoop()
{
	if(!g_pD3DDevice)return;

	SetCamera();

	KeyBoard();												// Get keyboard input

	// Clear the back buffer to a blue color
	g_pD3DDevice->Clear( 0, 
						 NULL, 
						 D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
						 D3DCOLOR_XRGB(0,20,155), 
						 1.0f, 
						 0 );


	g_pD3DDevice->BeginScene();

	DrawShape();											// Create & Render Shape

	D3DXVECTOR3 hitP;										// Intersection Point

	g_collision = TriangleLineCollision( g_line.p0,
										 g_line.p1,
										 g_tri.p0,
										 g_tri.p1,
										 g_tri.p2,
										 &hitP);

	static debugsphere ball(g_pD3DDevice);

	static debugtxt abc(g_pD3DDevice);
	abc.printf("Cursor Keys Move Line", 10, 30);
	abc.printf("Keyboard < & > To Move Line Into Z", 10, 45);
	abc.printf("LeftMouse Button and Drag To Move Cam", 10, 60);

	if (g_collision)
	{
		abc.printf("***Collision***", 10, 75);
		ball.Draw(g_pD3DDevice, hitP.x, hitP.y, hitP.z, 0.1f);
	}

	// Debug information - display what sort of Frames Per Second Where Getting
	FPSDisplay(g_pD3DDevice);

	g_pD3DDevice->EndScene();
	
	// After rendering the scene we display it.
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );


}//End GameLoop()

/***************************************************************************/

bool Create()
{
	//SetMaterial();

	SetLight();

	return true;
}// End Create(..)

/***************************************************************************/

bool Release()
{
	return true;
}// End Release(..)